; update 9/5/03 dmc
; commented much more of the code
; broke apart the DoVerlet function as follows:
;	broke out z axis velocity calculations to CalcVelocityZ
;	broke out the particle effects to DoParticle
;	broke out the car body align code
;	broke out the wheel spin code
;	deleted pivot steering method in favor of a true force applied to front wheels
;	broke out RelaxContraints() and extensively commented
;	broke out and completely rewrote the collapse protection code and extensively commented
;	broke out one updateworld call 
; completely rewrote DoVerlet and commented
; in type VerletPack deleted gravity.vector and traction
; deleted some functions obsolete functions
; deleted verletpack\part variables in favor of more generic code
; deleted obsolete globals
; simplified passed variables to DoParticle function
; added fully functional wind resistance
; fixed the wheel spin function so it uses correct math *i think*


Const ACTIVE=1					; collision type
Const VERLET=2					; collision Type

Const SIZE#=8					; just a multiplier for how big the Vobject will be


Type Vector

	Field x#
	Field y#
	Field z#

End Type


Type VerletPack							

	Field start_loc.vector					; Defines where this verlet started at so we can fix it when it gets to far out of place
	Field current.Vector						; Current position
	Field old.Vector						; Last position
	Field radius#							; Collider radius
	Field mass.vector						; Point mass of this particular verlet
	Field entity							; Pivot assigned to this verlet for both position and collision
	Field mesh							; Mesh assigned to this verlet if any such as wheels or body parts
	Field part								; Describes what verlet represents. example wheel or body part.
	Field state							; Collideable status true/false	
	Field contact_friction#					; When in contact with the ground how much friction is applied
	Field wheel_slip#[2]					; slip accumulator. needed to "smooth" out the slip value

End Type


Type Constraint							; Two verlets connected by a mutual spring

	Field v1.VerletPack						; First verlet
	Field v2.VerletPack						; Second verlet
	Field d#								; Distance between verlets

End Type


Type Vobject								; This is the parent object that the verlets are associated with

	Field v.VerletPack[8]					; The array of individual verlets
	Field num_verlets						; Total number of verlets (array dim value)
	Field player_pivot						; Pivot used to represent player. this will orient to the verlet structure
	Field player							; Mesh used to represent the player. this mesh is parented to player_pivot
	Field center.VerletPack					; Used to position player_pivot but not neccessarily parented to.
	Field x1.VerletPack						; x1 and x2 are used to define a vector left/right to align player_pivot
	Field x2.VerletPack
	Field z1.VerletPack						; z1 and z2 are used to define a vector forward/aft to align player_pivot
	Field z2.VerletPack
	Field X_vec#							; vectors for calculations
	Field Y_vec#
	Field Z_vec#
	Field mass#							; Mass for physics calculation on entire Vobject
	Field horsepower#						; Force for physics calculations on entire Vobject
	Field acceleration#						; Rate at which the Vobject accelerates
	Field velocity#							; a place to store how fast Vobject is going in the z direction(forward back)
	Field DragCoefficent#					; coefficient of drag for the Vobject
	Field CrossArea#						; rough cross sectional area for Vobject, use simple one dimensional value for now
End Type


Global vTemp1.Vector = Vector(0,0,0)	; Some temporary vectors for calculations
Global vTemp2.Vector = Vector(0,0,0)
Global GlobalForces.vector = Vector(0.0,9.8,0.0) ; Gravity. yes i know its a positive value. still dont know why this is but it works.
Global RelativeForces.vector = Vector(0,0,0) ;stores player generated forces
Global temp_turn#=0 ; variable used to pass player steering control to other functions
Global accelerator#=0 ; variable used to pass player gas pedal to other functions

; -------------------------------------------------------------------------------------------------------------------
; measure the start locations for each verlet so it can be fixed if it collapses. the position of 
; the verlet is stored in name\v[count]\start_loc.vector.
; -------------------------------------------------------------------------------------------------------------------
Function measure_pivots(name.Vobject)

	For count=0 To name\num_verlets
		
		EntityParent name\v[count]\entity, name\player_pivot
					
			name\v[count]\start_loc\x=EntityX(name\v[count]\entity)
			name\v[count]\start_loc\y=EntityY(name\v[count]\entity)
			name\v[count]\start_loc\z=EntityZ(name\v[count]\entity)

		EntityParent name\v[count]\entity, 0

	Next

End Function


; -------------------------------------------------------------------------------------------------------------------
; use this function to fix a collapsed Vobject. this is basically a reset button. does not prevent
; collapse. it only fixes it. uses name\v[count]\start_loc.vector to reset verlet positions.
; -------------------------------------------------------------------------------------------------------------------
Function fix_verlet(name.Vobject, tempheight)
	PositionEntity name\player_pivot, name\center\current\x, tempheight, name\center\current\z
	RotateEntity name\player_pivot,0,0,0
	
	For count=0 To name\num_verlets
		TFormPoint name\v[count]\start_loc\x, name\v[count]\start_loc\y, name\v[count]\start_loc\z, name\player_pivot, 0
		tempx# = TFormedX()
		tempy# = TFormedY()
		tempz# = TFormedZ()

		PositionEntity name\v[count]\entity, tempx, tempy, tempz

		name\v[count]\current\x = tempx
		name\v[count]\current\y = tempy
		name\v[count]\current\z = tempz

		name\v[count]\old\x = tempx
		name\v[count]\old\y = tempy
		name\v[count]\old\z = tempz	
	Next

End Function


; -------------------------------------------------------------------------------------------------------------------
; returns a "slip" value in the range of 0 (full traction) to .999 (full slip). this is to simulate
; static friction. 
; parameters:
; name.VObject, is the name of the verlet object
; verlet_element, is the current verlet within VObject
; threshold, is the value the verlet will begin to loose traction
; max, is the value the verlet will lose traction completely
; -------------------------------------------------------------------------------------------------------------------
Function wheel_force#(name.VObject, verlet_element.VerletPack, threshold#, max#)

	TFormPoint EntityX(verlet_element\entity,True), EntityY(verlet_element\entity,True), EntityZ(verlet_element\entity,True), 0,  name\player_pivot

	tempx# = TFormedX()
	tempy# = TFormedY()
	tempz# = TFormedZ()

	force# = Abs(verlet_element\start_loc\x - tempx) - threshold
	
	If (force < 0) Then force=0
		
	temp# = Abs(threshold - max)

	verlet_element\wheel_slip[0] = ((force / Sqr(force * force + temp * temp)) + verlet_element\wheel_slip[1] + verlet_element\wheel_slip[2])/3
	verlet_element\wheel_slip[2] = verlet_element\wheel_slip[1]
	verlet_element\wheel_slip[1] = verlet_element\wheel_slip[0]

	Return Sqr(verlet_element\wheel_slip[0])

End Function


; -------------------------------------------------------------------------------------------------------------------
; calculate and store the z axis (forward/back) velocity of VObject. uses the center verlet to
; derive velocity relative to VObjects Z axis (player_pivot).
; -------------------------------------------------------------------------------------------------------------------
Function CalcVelocityZ(name.Vobject, center_element.VerletPack)
			
			; where it is now.....
			TFormVector center_element\current\x, center_element\current\y,  center_element\current\z, 0, name\player_pivot

			;temptempx#= TFormedX() not needed 
			;temptempy#= TFormedY() not needed
			temptempz#= TFormedZ()
			
			; where it was....
			TFormVector center_element\old\x, center_element\old\y,  center_element\old\z, 0, name\player_pivot

			;tempvoldx# = TFormedX() not needed
			;tempvoldy# = TFormedY() not needed
			tempvoldz# = TFormedZ()

			If Abs(tempvoldz - temptempz) < 0.01 ; just a tolerance
				name\velocity = 0 
			Else 
				name\velocity = (tempvoldz - temptempz) ; store it off
			EndIf
			
End Function


; -------------------------------------------------------------------------------------------------------------------
; create partical effects for any verlet when in contact with any collideable entity
; parameters:
; name.Vobject, name of the VObject
; verlet_element, which verlet in VObject
; contact_surface, which entity to check for contact
; accelerator#, player controlled accelerator value
; -------------------------------------------------------------------------------------------------------------------
Function DoParticle(verlet_element.VerletPack, contact_surface, accelerator#)
	
	OnGround = 0
	For n = 1 To CountCollisions(verlet_element\entity)
		If (CollisionEntity(verlet_element\entity, n) = flr) OnGround = 1
	Next
	
	If (OnGround) ; If any of the tires is colliding with the ground...
	
		; Randomly spawn a dust particle, with a greater chance the more acceleration is occuring
		; or something.
		chance# = Abs(accelerator) * 0;.4
				
		If (Rnd(0,1) < chance)

			d.Dust = New Dust
			d\Sprite = CopyEntity(DustSprite)
			d\SprRot = Rnd(0,360)
			e = verlet_element\entity
			PositionEntity(d\Sprite, EntityX(e,1), EntityY(e,1), EntityZ(e,1))
			d\VelX = Rnd(-1, 1)
			d\VelY = Rnd(0.3, 0.7)
			d\VelZ = Rnd(-1, 1)
			d\Alpha = 1
			d\FadeRate = Rnd(0.01, 0.05)
			d\Size = 6
			d\Growth = Rnd(0.6, 2)
			d\VelR = Rnd(-2, 2)
			d\Friction = 0

		EndIf

	EndIf
	
End Function


; -------------------------------------------------------------------------------------------------------------------
; aligns the player_pivot to the verlet structure, also aligns all verlet pivots so the meshes
; tied to them will be in the proper orientation.
; -------------------------------------------------------------------------------------------------------------------
Function AlignPlayerPivot(name.Vobject)

		PositionEntity name\player_pivot, EntityX(name\center\entity), EntityY(name\center\entity), EntityZ(name\center\entity)

		name\X_vec = EntityX(name\x1\entity) - EntityX(name\x2\entity)
		name\Y_vec = EntityY(name\x1\entity) - EntityY(name\x2\entity)
		name\Z_vec = EntityZ(name\x1\entity) - EntityZ(name\x2\entity)

		AlignToVector name\player_pivot, name\X_vec, name\Y_vec, name\Z_vec, 1

		For count=0 To name\num_verlets
			AlignToVector name\v[count]\entity, name\X_vec, name\Y_vec, name\Z_vec, 1
		Next

		name\X_vec = EntityX(name\z1\entity) - EntityX(name\z2\entity)
		name\Y_vec = EntityY(name\z1\entity) - EntityY(name\z2\entity)
		name\Z_vec = EntityZ(name\z1\entity) - EntityZ(name\z2\entity)

		AlignToVector name\player_pivot, name\X_vec, name\Y_vec, name\Z_vec, 3
		
End Function


; -------------------------------------------------------------------------------------------------------------------
; simply spins the wheel tied to verlet_element as the car moves at forward_velocity
; -------------------------------------------------------------------------------------------------------------------
Function SpinWheel(name.Vobject, verlet_element.VerletPack, forward_velocity#)
	
	If forward_velocity <> 0 Then tempspin# = (forward_velocity*-100) / (Pi*verlet_element\radius)  Else tempspin#=0
	TurnEntity verlet_element\entity, tempspin, 0, 0, False 	

End Function


; -------------------------------------------------------------------------------------------------------------------
; Very much still work in progress. this needs to be called so all the position changes in the
; individual verlets will be check against the default distance set in each constraint. This is
; the passive portion of the physics where the entire structure shares forces between all
; the verlets. originally it was set to do it one direction (direction meaning from first constraint
; to last contraint) as many times as was set in ITERATIONS. I found that doing it in only one
; direction wasnt evenly distributing the passive forces. so the new method im testing is to
; first do it one direction (c\v1 TO c\v2) then reverse direction (c\v2 TO c\v1) so it better 
; balances the passive forces. both methods are included but one is commented out.
; -------------------------------------------------------------------------------------------------------------------
Function RelaxContraints()

	; original method
	;For n = 1 To ITERATIONS
		;setDistance(c\v1, c\v2, c\d)
	;next
	
	;new method, one pass forward one pass backwards
	For c.Constraint = Each Constraint
		setDistance(c\v1, c\v2, c\d)
		setDistance(c\v2, c\v1, c\d)
	Next

End Function


; -------------------------------------------------------------------------------------------------------------------
; this is pretty complex so youll just need to study it.
; parameters:
; name.VObject, is the name of the Vobject
; min_value#, this distance value is the danger point where a collapse MAY take place soon.
; max_value#, this distance value is the point at which a collapse has surely taken place.
; how many times the structure is looped thru the relaxation, collision update routine.
; -------------------------------------------------------------------------------------------------------------------
Function CollapseProtection(name.VObject, min_value#, max_value#, iterations)
	
	; first get the player pivot updated so it is loacted precisely at the objects center of gravity
	PositionEntity name\player_pivot, EntityX(name\center\entity),EntityY(name\center\entity),EntityZ(name\center\entity)
	
	; start to loop thru the verlets
	For count=0 To name\num_verlets
		
		;transform starting locations of verlets so it is relative to the global axis
		TFormPoint name\v[count]\start_loc\x, name\v[count]\start_loc\y, name\v[count]\start_loc\z, name\player_pivot, 0
		;record where our collision entities are and measure distance from start locations
		tempx# = TFormedX() - EntityX(name\v[count]\entity, True)
		tempy# = TFormedY() - EntityY(name\v[count]\entity, True)
		tempz# = TFormedZ() - EntityZ(name\v[count]\entity, True)
		;get a single distance value
		deltalength# = Sqr((tempx * tempx) + (tempy * tempy) + (tempz * tempz))
		
		;check for collapse
		If deltalength > max_value Then
			For count2=0 To name\num_verlets
				TFormPoint name\v[count2]\start_loc\x, name\v[count2]\start_loc\y, name\v[count2]\start_loc\z, name\player_pivot, 0
				tempx# = TFormedX()
				tempy# = TFormedY()
				tempz# = TFormedZ()
				
				;reset the verlets
				PositionEntity name\v[count2]\entity, tempx, tempy, tempz
				UpdateWorld
				;bleed off excess energy stored by making old = where the collision entities are
				name\v[count2]\old\x = EntityX(name\v[count2]\entity)
				name\v[count2]\old\y = EntityY(name\v[count2]\entity)
				name\v[count2]\old\z = EntityZ(name\v[count2]\entity)
			Next
			
		End If
		
		; check for collapse danger
		If deltalength > min_value Then
			; do loop as many times as set by the passed iterations value
			For count2=1 To iterations
				; update our current location varaibles so they exactly match where the collision entity is
				For count3=0 To name\num_verlets
					name\v[count3]\current\x = EntityX(name\v[count3]\entity)
					name\v[count3]\current\y = EntityY(name\v[count3]\entity)
					name\v[count3]\current\z = EntityZ(name\v[count3]\entity)
				Next
				;relax it once forward, once backwards
				For c.Constraint = Each Constraint
					setDistance(c\v1, c\v2, c\d)
					setDistance(c\v2, c\v1, c\d)
				Next
				; reposition our collision entities to be exactly where the math says they should be.
				For count3=0 To name\num_verlets
					PositionEntity name\v[count3]\entity, name\v[count3]\current\x, name\v[count3]\current\y, name\v[count3]\current\z
				Next
				; activate collision detection so the collision entities "pop" to the collideable surfaces
				UpdateWorld
			Next
					
			Else
				name\v[count]\current\x = EntityX(name\v[count]\entity)
				name\v[count]\current\y = EntityY(name\v[count]\entity)
				name\v[count]\current\z = EntityZ(name\v[count]\entity)	
		End If
		
	Next
	
	For c.Constraint = Each Constraint
		setDistance(c\v1, c\v2, c\d)
		setDistance(c\v2, c\v1, c\d)
	Next
End Function


; -------------------------------------------------------------------------------------------------------------------
; this reads the final position after all collisions of the actuall entity tied to the verlet
; coordinates and assigns it to the position variable.
; -------------------------------------------------------------------------------------------------------------------
Function FinalizeVelertPosition(name.Vobject)
	
	For count = 0 To name\num_verlets
		
		name\v[count]\current\x = EntityX(name\v[count]\entity)
		name\v[count]\current\y = EntityY(name\v[count]\entity)
		name\v[count]\current\z = EntityZ(name\v[count]\entity)

	Next

End Function

; -------------------------------------------------------------------------------------------------------------------
; pretty self explanatory
; -------------------------------------------------------------------------------------------------------------------
Function WindResist#(name.Vobject, verlet_element.VerletPack, GlobalFriction#, temp_velocity#)
	wr#= -(.5 * name\DragCoefficent * GlobalFriction *name\CrossArea * (temp_velocity * temp_velocity) * Sgn(temp_velocity))
	If Abs(wr) < .001 Then wr=0
	Return wr
End Function


; -------------------------------------------------------------------------------------------------------------------
; computes all forces exerted on each verlet.
; parameters:
; name.Vobject, is the name of the of the VObject
; verlet_element, which verlet
; GlobalForces.vector, xyz forces that act globally on the structure. example down is always
; down as in gravity, all global forces need to be transformed as shown below.
; rfox,rfoy,rfoz are the relative forces acting on this verlet. relative meaning that z is forward
; to the structure not global z
; globalfriction# is the air density. global friction needs no transformationor vector. it is directionless.
; rfrx, rfry, rfrz are the relative frictions applied to this verlet. this is how we achieve static
; friction on a tire for example.
; -------------------------------------------------------------------------------------------------------------------
Function DoVerlet(name.Vobject, verlet_element.VerletPack, GlobalForces.vector, rfox#, rfoy#, rfoz#, GlobalFriction#, rfrx#, rfry#, rfrz#)
	
	; transform from global to player_pivot		
	TFormVector verlet_element\current\x, verlet_element\current\y, verlet_element\current\z, 0, name\player_pivot
				
		; stored version of current position
		temptempx# = TFormedX()
		temptempy# = TFormedY()
		temptempz# = TFormedZ()
				
		; current position
		tempvmx# = TFormedX()
		tempvmy# = TFormedY()
		tempvmz# = TFormedZ()

	; transform from global to player_pivot
	TFormVector verlet_element\old\x, verlet_element\old\y, verlet_element\old\z, 0, name\player_pivot
				
		; old position
		tempvoldx# = TFormedX()
		tempvoldy# = TFormedY()
		tempvoldz# = TFormedZ()
		
	If (CountCollisions(verlet_element\entity) > 0) Then
		;set up temp relative frictions and forces
			
		;relative force
		tempRelForceX# = rfox
		tempRelForceY# = rfoy
		tempRelForceZ# = rfoz
			
		;relative friction
		tempRelFrictX# = rfrx
		tempRelFrictY# = rfry
		tempRelFrictZ# = rfrz
			
		tempContactFrict# = 1- verlet_element\contact_friction
		
		Else 
			tempRelForceX# = 0 ; 0 means no force
			tempRelForceY# = 0
			tempRelForceZ# = 0
				
			tempRelFrictX# = 1 ;1 means no friction
			tempRelFrictY# = 1
			tempRelFrictZ# = 1
				
			tempContactFrict# = 1 ;1 means no friction
	End If
			
	; transform from global to player_pivot		
	TFormVector GlobalForces\x, GlobalForces\y,  GlobalForces\z, 0, name\player_pivot
			
		;global force
		tempGFx# = TFormedX()
		tempGFy# = TFormedY()
		tempGFz# = TFormedZ()
	
	;set up wind resistance	
	tempWindX# = WindResist(name.Vobject, verlet_element.VerletPack, GlobalFriction, (tempvmx - tempvoldx))
	tempWindy# = WindResist(name.Vobject, verlet_element.VerletPack, GlobalFriction, (tempvmy - tempvoldy))
	tempWindz# = WindResist(name.Vobject, verlet_element.VerletPack, GlobalFriction, (tempvmz - tempvoldz))
		
	; heres the formula that does the physics
	tempvmx = tempvmx - ((tempvoldx * tempContactFrict * tempRelFrictX)  + ((tempGFx + tempRelForceX - tempWindX) * (DeltaTime * DeltaTime))) + (temptempx * tempContactFrict * tempRelFrictX) 
	tempvmy = tempvmy - ((tempvoldy * tempContactFrict * tempRelFrictY)  + ((tempGFy + tempRelForceY - tempWindY) * (DeltaTime * DeltaTime))) + (temptempy * tempContactFrict * tempRelFrictY) 
	tempvmz = tempvmz - ((tempvoldz * tempContactFrict * tempRelFrictZ)  + ((tempGFz + tempRelForceZ - tempWindZ) * (DeltaTime * DeltaTime))) + (temptempz * tempContactFrict * tempRelFrictZ)   

	; old is updated to current		
	tempvoldx = temptempx
	tempvoldy = temptempy
	tempvoldz = temptempz

	; transform from player_pivot back to global
	TFormVector tempvmx, tempvmy, tempvmz, name\player_pivot, 0

		verlet_element\current\x = TFormedX()
		verlet_element\current\y = TFormedY()
		verlet_element\current\z = TFormedZ()
	
	; transform from player_pivot back to global
	TFormVector tempvoldx, tempvoldy,  tempvoldz, name\player_pivot, 0

		verlet_element\old\x = TFormedX()
		verlet_element\old\y = TFormedY()
		verlet_element\old\z = TFormedZ()
	
	; move everything to its new position
	PositionEntity verlet_element\entity, verlet_element\current\x, verlet_element\current\y, verlet_element\current\z

End Function


; -------------------------------------------------------------------------------------------------------------------
; checks the distance between constraint 1 and constraint 2 against the stored value it should
; be. then it attempts to move them apart to more closely match the stored distance.
; -------------------------------------------------------------------------------------------------------------------
Function SetDistance(v1.VerletPack,v2.VerletPack,dist#)	

	vtemp1\x = v1\current\x - v2\current\x
	vtemp1\y = v1\current\y - v2\current\y
	vtemp1\z = v1\current\z - v2\current\z

	deltalength# = Sqr((vTemp1\x * vTemp1\x) + (vTemp1\y * vTemp1\y) + (vTemp1\z * vTemp1\z))

	; It is impossible to get a negative number with a square root.
	; Might want to do a special case for the calculations if the length is 0 instead of fudging it.
	If (deltalength <= 0.0) Then deltalength = 0.0000001

	diff# = (deltalength - dist) / deltalength
	
	txmass# =  v1\mass\x + v2\mass\x
	tymass# =  v1\mass\y + v2\mass\y
	tzmass# =  v1\mass\z + v2\mass\z

	v1\current\x=v1\current\x - vtemp1\x * (diff * (v2\mass\x / txmass)) * (diff+1)
	v1\current\y=v1\current\y - vtemp1\y * (diff * (v2\mass\y / tymass)) * (diff+1)
	v1\current\z=v1\current\z - vtemp1\z * (diff * (v2\mass\z / tzmass)) * (diff+1)

	v2\current\x=v2\current\x + vtemp1\x *(diff * (v1\mass\x / txmass))  / (diff+1)
	v2\current\y=v2\current\y + vtemp1\y *(diff * (v1\mass\y / tymass))  / (diff+1)
	v2\current\z=v2\current\z + vtemp1\z *(diff * (v1\mass\z / tzmass))  / (diff+1)

End Function


; -------------------------------------------------------------------------------------------------------------------
; initalizes a verlet
; parameters:
; x,y,z are the start coords
; radius is the radius of the collision entity
; massx, massy, massz are the mass values for each axis - using different values on each axis
; to get some control on vehicle handling characteristics.
; -------------------------------------------------------------------------------------------------------------------
Function Verlet.VerletPack(x#,y#,z#,radius#, massX#, massY#, massZ#, mesh, state, collision_type, contact_friction#)

	v.VerletPack = New VerletPack

	v\current = Vector(x,y,z) ; current location
	v\old = Vector(x,y,z) ; old location
	v\start_loc = Vector(0,0,0) ; initalize with zeros to be filled later
	v\mass = Vector(0,0,0) ; initialize mass
	
	If (radius <= 0.0) radius = 0.01
		
	v\radius = radius ;collision radius

	v\mass\x = massX#
	v\mass\y = massY#
	v\mass\z = massZ#

	v\entity = CreatePivot() ;used as the collision element

	PositionEntity v\entity,x,y,z,True

	v\mesh = mesh ;mesh assigned if any

	If (Mesh <> 0) 
		EntityParent v\mesh, v\entity,True
	EndIf

	v\state = state	 ;true for collideable, false for non collideable

	If v\state=True
		EntityType v\entity, collision_type
		EntityRadius v\entity,radius#
	EndIf

	v\contact_friction = contact_friction# ;rolling friction
	
	Return v

End Function


; -------------------------------------------------------------------------------------------------------------------
; this creates a logical link between 2 verlets that are consider connected. the distance
; between the tow verlets is recorded so later on when we call setdistance we can push
; the verlets apart so the structure keeps its form.
; -------------------------------------------------------------------------------------------------------------------
Function Constraint2.Constraint(v1.VerletPack,v2.VerletPack)

	c.Constraint = New Constraint

	c\v1 = v1 ;verlet 1
	c\v2 = v2 ;verlet 2 
	
	; store how far they are apart
	c\d = Sqr((v2\current\x - v1\current\x)^2 + (v2\current\y - v1\current\y)^2 + (v2\current\z - v1\current\z)^2)
	
	; You may need this someday ... we don't for what we have here
	;Return c	

End Function


; -------------------------------------------------------------------------------------------------------------------
; Creates a vector.
; -------------------------------------------------------------------------------------------------------------------
Function Vector.Vector(x#=0.0,y#=0.0,z#=0.0)

	v.Vector = New Vector
	
	v\x=x
	v\y=y
	v\z=z

	Return v

End Function 


; -------------------------------------------------------------------------------------------------------------------
; ??? beats me also, it might be al's stuff. dmc
; -------------------------------------------------------------------------------------------------------------------
Function CloseAngle(source#, target#)

	While (source < target - 180)
		source = source + 360
	Wend

	While (source > target + 180)
		source = source - 360
	Wend

	Return source

End Function