; update 9/5/03 dmc
; deleted mouse vehicle control to simplify control scheme.
; fixed a bug where "flr" was being called out globally in a gosub loop
; changed naming convention for verlets to be more intuitive
; deleted obsolete globals
; tweaked steering functions
; fixed bug where wheels mesh were being parented to none existant parent.
; fixed reverse steering bug
; added a large list of tuning variables for the car.


Include "verletlib-002.bb"

; graphics settings
Const WIDTH=1024
Const HEIGHT=768

;car tuning variables

;steering
Const maxturn=150;130 ;maximum turn force, how tight you can turn. higher = sharper
Const TurnRate#=8;6; how fast you can achieve max turn rate. higher = faster
Const TurnReturn#=2;1.03 ;how fast the steering returns to nuetral. a value of 1 will leave the steering at last value. higher for return to nuetral
;gas pedal
Const SnapResponse#=.1 ;value from 0 to 1. how fast max gas pedal is reached
Const GasReturn#=1.03 ; how fast the gas returns to nuetral. a value of 1 will leave the gas at last value. higher for return to nuetral
;suspension
Const front_stiff = 20 ; This is actually the mass value for the front wheel. lower mass = suspension less stiff. caution will collapse
Const rear_stiff = 20 ; This is actually the mass value for the rear wheel. lower mass = suspension less stiff. caution will collapse
;tire slip
Const FrontSlipMin# = 2;.9 ;the value at which the front tires will begin to slip
Const FrontSlipMax# = 4;1.2 ;the value at which the front tires will slip completely
Const RearSlipMin# = 2;1 ;the value at which the rear tires will begin to slip
Const RearSlipMax# = 4;1.5 ;the value at which the rear tires will slip completely
;tire size
Const FrontTireSize# = 5; this the verlet collision radius
Const RearTireSize# = 5; this the verlet collision radius
;rolling friction for tires
Const RollingFriction#=.02; how much friction on the tires when they are in contact with the ground
;body friction
Const BodyFriction#=.07;how much friction on the body when they are in contact with the ground
;wind resist
Const AirDensity# = .7 ; density of air for wind resist function
Const DC# = .3 ;Drag coefficient. how streamlined the car shape. used for wind resist
Const CA# = .5 ;Cross sectional area. how large an area the wind is running across. used for wind resistance
;down force
Const DownForce# = 1 ;how much down force is allowed. value from 0 to 1. zero= no down force, 1=max downforce
;power of engine
Const VM = 10 ;Vehicle Mass. Fictional mass value used for acceleration calculations on player_pivot.
Const HP = 1500;1000 ;Horse Power. Fictional force value exerted on z axis, local to pivot, of verlet for acceleration calculations


Type Dust

	Field Sprite				
	Field VelX#, VelY#, VelZ#	
	Field Alpha#				
	Field FadeRate#				
	Field Size#, VelR#, SprRot#, Growth#, Friction#

End Type


Global DeltaTime#					; Time between physics calculations
Global DustSprite
Global car.Vobject
Global BkgObjs
Global Sky
Global flr
Global cam1
Global cam2
Global Camera
Global CarInterior


; -------------------------------------------------------------------------------------------------------------------
.Main

	Gosub Initialize_Game

	HidePointer

	;display directions

		While Not KeyHit(57)

			RenderWorld

			Text 20,20, "Tab changes view."
			Text 20,35, "Arrow keys control vehicle.
			Text 20,50, "Escape exits program."
			Text 20,65, "Space bar resets vehicle"
			Text 20,80, "Hit space bar to begin."

			Flip False

		Wend

	DeltaTime#=.09
	fRate = (1000/72) ;*set second number to desired frame rate
	view=0


	PrevCP# = 0
	PrevCY# = 0
	PrevCR# = 0

	CamHeight# = 20 ; camera height above player
	CamDist# = 150 ; camera dist from player
	CamLag# = .5 ; Tightness of camera to its target (1 = strongest)

	fps = createtimer (50)
	
	While Not KeyHit(1)
		
		waittimer (fps)

		time1 = MilliSecs() ;time at start of processing loop

		Gosub Get_Input
		Gosub Animate_Dust
		Gosub Animate_Camera

		; calc and store the Z velocity for this frame.
		CalcVelocityZ(car.Vobject, CG.VerletPack)
		
		; do dust effects
		DoParticle(RightFrontTire.VerletPack, flr, accelerator#)
		DoParticle(LeftFrontTire.VerletPack, flr, accelerator#)
		DoParticle(RightRearTire.VerletPack, flr, accelerator#)
		DoParticle(LeftRearTire.VerletPack, flr, accelerator#)

		; Update the verlets for this frame.
		DoVerlet(car.Vobject, CG.VerletPack, GlobalForces.vector, 0,0,0, AirDensity, 1,1,1)
		
		temp_sliprf#= wheel_force(car.vobject, RightFrontTire.VerletPack, FrontSlipMin, FrontSlipMax)
		DoVerlet(car.Vobject, RightFrontTire.VerletPack, GlobalForces.vector, RelativeForces\x,RelativeForces\y,RelativeForces\z, AirDensity, temp_sliprf#,1,1)

		temp_sliplf#= wheel_force(car.vobject, LeftFrontTire.VerletPack, FrontSlipMin, FrontSlipMax)
		DoVerlet(car.Vobject, LeftFrontTire.VerletPack, GlobalForces.vector, RelativeForces\x,RelativeForces\y,RelativeForces\z, AirDensity, temp_sliplf#,1,1)
	
		temp_sliprr#= wheel_force(car.vobject, RightRearTire.VerletPack, RearSlipMin, RearSlipMax)
		DoVerlet(car.Vobject, RightRearTire.VerletPack, GlobalForces.vector, 0,RelativeForces\y,RelativeForces\z, AirDensity, temp_sliprr#,1,1)

		temp_sliplr#= wheel_force(car.vobject, LeftRearTire.VerletPack, RearSlipMin, RearSlipMax)
		DoVerlet(car.Vobject, LeftRearTire.VerletPack, GlobalForces.vector, 0,RelativeForces\y,RelativeForces\z, AirDensity, temp_sliplr#,1,1)
		
		DoVerlet(car.Vobject, RightFrontRoof.VerletPack, GlobalForces.vector, 0,RelativeForces\y,0, AirDensity, 1,1,1)
		DoVerlet(car.Vobject, LeftFrontRoof.VerletPack, GlobalForces.vector, 0,RelativeForces\y,0, AirDensity, 1,1,1)
		DoVerlet(car.Vobject, RightRearRoof.VerletPack, GlobalForces.vector, 0,RelativeForces\y,0, AirDensity, 1,1,1)
		DoVerlet(car.Vobject, LeftRearRoof.VerletPack, GlobalForces.vector, 0,RelativeForces\y,0, AirDensity, 1,1,1)
		
		UpdateWorld
				
		; very crude protection against structure collapsing
		CollapseProtection(car.Vobject,5,40,1)

		; aligns pivots and meshes to verlet structure
		AlignPlayerPivot(car.Vobject)
		
		; make wheels spin
		SpinWheel(car.Vobject, RightFrontTire.VerletPack, car\velocity)
		SpinWheel(car.Vobject, LeftFrontTire.VerletPack, car\velocity)
		SpinWheel(car.Vobject, RightRearTire.VerletPack, car\velocity)
		SpinWheel(car.Vobject, LeftRearTire.VerletPack, car\velocity)
		
		; make sure variables match actual location		
		FinalizeVelertPosition(car.Vobject)

		; does the passive physics and relaxes the structure
		; this must be called last.
		RelaxContraints()
	        
		; Make the sky and the background objects stay centered on the camera.
		PositionEntity BkgObjs, EntityX(Camera, 1), EntityY(Camera, 1), EntityZ(Camera, 1)
		PositionEntity Sky, EntityX(Camera, 1), 0, EntityZ(Camera, 1)

		RenderWorld
		Text 20,20, "FORCES ON CAR BODY"
		Text 40,35, "Forward Velocity: "+car\velocity
		Text 40,50, "Downforce: "+RelativeForces\y
		Text 40,65, "Wind Resistance (X Axis Global): " + WindResist(car.Vobject, CG.VerletPack, AirDensity, (CG\current\x - CG\old\x))
		Text 40,80, "Wind Resistance (Y Axis Global): " + WindResist(car.Vobject, CG.VerletPack, AirDensity, (CG\current\y - CG\old\y))
		Text 40,95, "Wind Resistance (Z Axis Global): " + WindResist(car.Vobject, CG.VerletPack, AirDensity, (CG\current\z - CG\old\z))
		Text 20,110, "WHEEL SLIP"
		Text 40,125, "Right Front: " +  Int(Sqr(RightFrontTire\wheel_slip[0])*100)
		Text 40,140, "Left Front: " +  Int(Sqr(LeftFrontTire\wheel_slip[0])*100)
		Text 40,155, "Right Rear: " +  Int(Sqr(RightRearTire\wheel_slip[0])*100)
		Text 40,170, "Left Rear: " +  Int(Sqr(LeftRearTire\wheel_slip[0])*100)
		Delay fRate - (MilliSecs() - time1) ;delay execution            (Does not work in Texas!)	
	
		Flip False

	Wend

End


; -------------------------------------------------------------------------------------------------------------------
.Get_Input

		If KeyDown(205) ;turn the car
			
			temp_turn=(temp_turn + TurnRate)
			If (Abs(temp_turn) > maxturn) Then temp_turn = maxturn * Sgn(temp_turn)

		Else 
			
			If KeyDown(203) ;turn the car

				temp_turn=(temp_turn - TurnRate)
				If temp_turn < -maxturn Then temp_turn = -maxturn

			Else ; If the player isn't providing any turning input...

				temp_turn=temp_turn/TurnReturn ;slowly return steering to neutral position over time
								
				; If the car is pretty much driving straight, then make it drive exactly straight.
				If (temp_turn > -0.01) And (temp_turn < 0.01) Then temp_turn=0		
				
			EndIf

		EndIf

		; if car is moving slowly limit turn radius until temp_turn=0 if velocity=0
		If Abs(car\velocity) <= 2.5 Then slow_limit# = Abs(car\velocity) / 2.5	
		
		RelativeForces\x = temp_turn * slow_limit# * Sgn(car\velocity) / Sqr(Sqr(Abs(car\velocity *.01)+1)) ;important! this is passed to the verlets that are the front tires
		RelativeForces\y=(Abs(car\velocity)^2) * DownForce ;downforce calculations

		If KeyDown(200) 

			accelerator=accelerator + SnapResponse
			If (accelerator > 1) Then accelerator = 1

		Else 
			
			If KeyDown(208) 

				accelerator=accelerator - SnapResponse
				If (accelerator < -1) Then accelerator = -1

			Else

				accelerator=accelerator/GasReturn ;slowly return to zero
				If (accelerator > -0.01) And (accelerator < 0.01) Then accelerator=0

			EndIf
		
		EndIf
		
		;important! this is passed to the verlets that are the rear wheels
		RelativeForces\z = accelerator * car\horsepower / car\mass

		; If the player hits space, set gravity to 0 for this frame, and fix the car so it's level with the
		; ground again.
		If KeyDown(57) Then 
			GlobalForces.vector = Vector(0.0,0.0,0.0)
			fix_verlet(car.Vobject, 100) ; the number is how high above ground the car will appear

		Else 

			; Otherwise, set gravity to be normal this frame.
			GlobalForces.vector = Vector(0.0,9.8,0.0)

		EndIf

		
		; Select the camera view.
		If KeyHit(15)  

			view=view+1
			If (view >=3) Then view=0

			If (view = 0) Or (view = 2) ; Chase
			
				CameraZoom(Camera, 1.5)
				EntityParent(Camera, Cam2)
				RotateEntity(Camera, 0, 0, 0)
				HideEntity(RearViewCam)
				EntityOrder(CarInterior, 0)
				HideEntity(RearViewBlock)
		
				If (view = 0)

					CamHeight# = 20 ; camera height above player
					CamDist# = 150 ; camera dist from player
					CamLag# = .5 ; Tightness of camera to its target (1 = strongest)

				Else

					CamHeight# = 30 ; camera height above player
					CamDist# = 70 ; camera dist from player
					CamLag# = 0.01 ; Tightness of camera to its target (1 = strongest)

				EndIf

			Else
				
				If (view = 1) ; Internal

					CameraZoom(Camera, 1)
					EntityParent(Camera, Cam1)
					RotateEntity(Camera, 0, 180, 0)
					ShowEntity(RearViewCam)
					EntityOrder(CarInterior, -1)
					ShowEntity(RearViewBlock)

				EndIf

			EndIf

			; I'm not sure why the camera is being positioned at 0,0,0.  I assume it must be parented to a pivot or
			; something which is attached to the car,
			PositionEntity(Camera, 0, 0, 0)

		EndIf

Return


; -------------------------------------------------------------------------------------------------------------------
.Initialize_Game


	Graphics3D WIDTH, HEIGHT;, 0, 2

	; Make the car.
		Gosub Make_Car


	; Enable collisions between the verlets and "ACTIVE" whatever that is.  ACTIVE's are collideable objects in the world
		Collisions VERLET, ACTIVE, 2, 2

	
	; Create a pivot to parent the background objects to, I guess. 
	; I don't know why the background objects are being parented to anything.
		BkgObjs = CreatePivot()


	; Create lights.
		SunLight = CreateLight()
		RotateEntity SunLight, 50, 180+40, 0
		LightColor SunLight, 224, 221, 203

		BakLight = CreateLight()
		RotateEntity BakLight, 20, 40, 0
		LightColor BakLight, 94, 104, 103
	
		AmbientLight 64, 59, 48

	
	; Create sun flare.
		flare = LoadSprite("flare.png")
	
		EntityParent(flare, BkgObjs)
		RotateEntity(flare, 50, 180+40, 0)
		MoveEntity (flare, 0, 0, -8000)
		ScaleSprite(flare, 5000, 5000)
		EntityColor(flare, 255-128, 255-176, 255-235)
		SpriteViewMode(flare, 3)


	; Create sky.
		Sky = LoadMesh("sky.B3D")
		
		ScaleEntity(Sky, 3.8*9500, 5.6*9500, 3.8*9500)
		EntityFX(Sky, 1)

	
	; Create dust particles.		
		DustSprite = LoadSprite("dust.png", 3)
		SpriteViewMode(DustSprite, 3)
		EntityBlend(DustSprite, 1)
		HideEntity(DustSprite)

	
	; Create cameras.


		; No idea what these are for.

			camnul = CreatePivot()
			camtarget = CreatePivot()


		; In car view camera?

			cam1 = CreatePivot(car\player)
			PositionEntity (cam1, 2.8, 0.4, -1.8)
			RotateEntity (cam1, 0, 180, 0)


		; Chase camera?

			cam2 = CreatePivot()
			Camera = CreateCamera(cam2)
			CameraZoom(Camera, 1.5)
			CameraClsColor Camera,128,176,235
			CameraRange Camera,0.5,10000
			PositionEntity (Camera, 0, 10, 0)

	
		; Rear view mirror.

			RearViewCam = CreateCamera(car\player)
			PositionEntity (RearViewCam, 0.5, 0.5, 10)
			TurnEntity (RearViewCam, 0, 0, 0)
			CameraZoom(RearViewCam, 0.7)
			CameraClsColor RearViewCam,128,176,235
			CameraRange RearViewCam,0.5,10000
			CameraViewport (RearViewCam, 0.55*ScrWidth, 0.05*ScrHeight, 0.4*ScrWidth, 0.15 * ScrHeight)
			HideEntity(RearViewCam)

			; No idea what this is for:
			
			RearViewBlock = CreateCube(Camera)
			vratio# = 1.0 * ScrHeight / ScrWidth
			EntityColor(RearViewBlock, 0, 0, 0)
			PositionEntity(RearViewBlock, 7.65, 11.5 * vratio, 10.7)
			ScaleEntity(RearViewBlock, 6.3, 2.5 * vratio, 0.00001)
			EntityOrder(RearViewBlock, -1)
			HideEntity(RearViewBlock)


	; Create track.	

		track = LoadMesh("track.B3D")
		ScaleEntity (track, 70, 70, 70)
		EntityType (track, ACTIVE)
	
		ground_tex = LoadTexture("mud.png")
		ScaleTexture ground_tex,500,500
		flr=CreatePlane()

		EntityType flr,ACTIVE
		EntityTexture flr,ground_tex
		EntityFX flr,1
		PositionEntity flr,0,0,0


	; set the tire mesh scale to the verlets collision radius value and put in position

		ScaleEntity car\v[1]\mesh,car\v[1]\radius,car\v[1]\radius/2,car\v[1]\radius
		RotateEntity car\v[1]\mesh,0,0,90
		PositionEntity car\v[1]\mesh,0,0,0

		ScaleEntity car\v[2]\mesh,car\v[2]\radius,car\v[2]\radius/2,car\v[2]\radius
		RotateEntity car\v[2]\mesh,0,0,90
		PositionEntity car\v[2]\mesh,0,0,0
	
		ScaleEntity car\v[5]\mesh,car\v[5]\radius,car\v[5]\radius/2,car\v[5]\radius
		RotateEntity car\v[5]\mesh,0,0,90
		PositionEntity car\v[5]\mesh,0,0,0

		ScaleEntity car\v[6]\mesh,car\v[6]\radius,car\v[6]\radius/2,car\v[6]\radius
		RotateEntity car\v[6]\mesh,0,0,90
		PositionEntity car\v[6]\mesh,0,0,0


Return


; -------------------------------------------------------------------------------------------------------------------
.Make_Car

	car.Vobject = New Vobject 			;initialize the Vobject type

	;start location of vehicle

		tempx# = 0							
		tempy# = 50				
		tempz# = 150			

	;this is the master pivot that is used to align the body, and other miscelaneous things
		car\player_pivot=CreatePivot()			

		
	car\player = LoadMesh("carbody.B3D", car\player_pivot)
	
	CarInterior = LoadMesh("carinterior.B3D", car\player)

	ScaleEntity(car\player, 2, 2, 2)
	PositionEntity car\player, 0, 2, -6

	car\num_verlets = 8	 ;total number of verlets in the Vobject. count from zero
	car\mass = VM ;this mass value is used for physics calculations on player_pivot.
	car\horsepower = HP ;the force exerted on z axis,local to pivot, of verlet
	car\DragCoefficent = DC ;how streamlined the car shape
	car\CrossArea = CA ;how large an area the wind is running across
	
	;just a multiplier for the Vobject size
		sx# = 1.9								
		sy# = 1.2
		sz# = 2.65

	; Create tires.

		; Copymesh should be used to create the other three tires, not loadmesh.
	
		tm = LoadMesh("tire.B3D")
		RightFrontTire.VerletPack = Verlet(tempx - SIZE * sx, tempy - SIZE * sy, tempz - SIZE * sz,  FrontTireSize, 100, front_stiff, 100, tm, True, VERLET, RollingFriction)

		tm = LoadMesh("tire.B3D")
		RotateMesh(tm, 180, 0, 0)

		LeftFrontTire.VerletPack = Verlet(tempx + SIZE * sx, tempy - SIZE * sy, tempz - SIZE * sz, FrontTireSize, 100, front_stiff, 100, tm, True, VERLET, RollingFriction)
		LeftFrontRoof.VerletPack = Verlet(tempx + SIZE * sx, tempy + SIZE * sy, tempz - SIZE * sz, 7, 400, 400, 400, 0, True, VERLET, BodyFriction)
		RightFrontRoof.VerletPack = Verlet(tempx - SIZE * sx, tempy + SIZE * sy, tempz - SIZE * sz, 7, 400, 400 ,400.0, 0, True, VERLET, BodyFriction)

		tm = LoadMesh("tire.B3D")

		RightRearTire.VerletPack = Verlet(tempx - SIZE * sx, tempy - SIZE * sy, tempz + SIZE * sz, RearTireSize, 100, rear_stiff, 100, tm, True, VERLET, RollingFriction)

		tm = LoadMesh("tire.B3D")
		RotateMesh(tm, 180, 0, 0)

		LeftRearTire.VerletPack = Verlet(tempx + SIZE * sx, tempy - SIZE * sy, tempz + SIZE * sz, RearTireSize, 100, rear_stiff, 100, tm, True, VERLET, RollingFriction)
		LeftRearRoof.VerletPack = Verlet(tempx + SIZE * sx, tempy + SIZE * sy, tempz + SIZE * sz, 7, 400, 400.0, 400, 0, True, VERLET, BodyFriction)
		RightRearRoof.VerletPack = Verlet(tempx - SIZE * sx, tempy + SIZE * sy, tempz + SIZE * sz, 7, 400, 400.0, 400, 0, True, VERLET, BodyFriction)
		CG.VerletPack = Verlet(tempx, tempy + 5, tempz + 8, 7, 400, 400.0, 400, 0, False, VERLET, 0)

	; Make springs between verlets to make car body rigid.

		; Edges
		
			Constraint2(RightFrontTire, LeftFrontTire)
			Constraint2(LeftFrontTire, LeftFrontRoof)
			Constraint2(RightFrontTire, RightFrontRoof)
			Constraint2(LeftFrontRoof, RightFrontRoof)
			Constraint2(RightRearTire, LeftRearTire)	
			Constraint2(LeftRearTire, LeftRearRoof)
			Constraint2(RightRearTire, RightRearRoof)
			Constraint2(LeftRearRoof, RightRearRoof)
			Constraint2(RightFrontTire, RightRearTire)
			Constraint2(LeftFrontTire, LeftRearTire)
			Constraint2(LeftFrontRoof, LeftRearRoof)
			Constraint2(RightFrontRoof, RightRearRoof)

		; Cross-faces

			Constraint2(RightFrontTire, LeftFrontRoof)
			Constraint2(LeftFrontTire, RightFrontRoof)
			Constraint2(RightRearTire, LeftRearRoof)
			Constraint2(LeftRearTire, RightRearRoof)
			Constraint2(RightFrontTire, LeftRearTire)
			Constraint2(RightFrontTire, RightRearRoof)
			Constraint2(LeftFrontTire, RightRearTire)
			Constraint2(LeftFrontTire, LeftRearRoof)
			Constraint2(LeftFrontRoof, LeftRearTire)
			Constraint2(LeftFrontRoof, RightRearRoof)
			Constraint2(RightFrontRoof, RightRearTire)
			Constraint2(RightFrontRoof, LeftRearRoof)

		; Diagonals

			Constraint2(RightFrontTire, LeftRearRoof)
			Constraint2(LeftFrontTire, RightRearRoof)
			Constraint2(LeftFrontRoof, RightRearTire)
			Constraint2(RightFrontRoof, LeftRearTire)


		; Constraining the center sphere

			Constraint2(RightFrontTire, CG)
			Constraint2(LeftFrontTire, CG)
			Constraint2(LeftFrontRoof, CG)
			Constraint2(RightFrontRoof, CG)
			Constraint2(RightRearTire, CG)
			Constraint2(LeftRearTire, CG)
			Constraint2(LeftRearRoof, CG)
			Constraint2(RightRearRoof, CG)


	; Assign individual verlets to the array	
	; the array is used to simplify counting thru the entire structure of verlets

		car\v[0] = CG
		car\v[1] = RightFrontTire
		car\v[2] = LeftFrontTire
		car\v[3] = LeftFrontRoof
		car\v[4] = RightFrontRoof
		car\v[5] = RightRearTire
		car\v[6] = LeftRearTire
		car\v[7] = LeftRearRoof
		car\v[8] = RightRearRoof

	
	; assign a verlet the center position
	; This is the cars center of mass

		car\center=CG

	
	; Set up variables used for player mesh/pivot aligntovector. one is left to right, other is forward back.

		car\x1=LeftFrontRoof
		car\x2=RightFrontRoof
		car\z1=LeftRearRoof
		car\z2=LeftFrontRoof


		RelaxContraints() ;relax everything before sim begins
		
		; move the master pivot to the cars center of mass and align the car body before sim begins
		PositionEntity car\player_pivot, EntityX(car\center\entity),EntityY(car\center\entity),EntityZ(car\center\entity)
		AlignPlayerPivot(car.Vobject)
	
		; store off the starting locations of relaxed verlets so we can use these value to detect if collaps has happened.
		measure_pivots(car.Vobject)

Return


.Animate_Dust

		; Animate dust particles.
		For dst.Dust = Each Dust

			dst\Alpha = dst\Alpha - dst\FadeRate
			dst\Size = dst\Size + dst\Growth

			If (dst\Alpha <= 0) Or (dst\Size <= 0)
				FreeEntity(dst\Sprite)
				Delete(dst)
			Else
				
				; Particles grow over time
				dsize# = dst\Size * 0.5
				ScaleSprite (dst\Sprite, dsize, dsize)

				; Fade particles over time
				alpha# = dst\Alpha
				If (alpha > 1) alpha = 1

				; Fade particles close to camera
				dist# = EntityDistance(dst\Sprite, Camera)
				If (dist < dsize) And (dist > 0) alpha = alpha * (dsize/dist)	
				EntityAlpha(dst\Sprite, alpha)

				; Rotate particles over time
				dst\SprRot = dst\SprRot + dst\VelR
				RotateSprite (dst\Sprite, dst\SprRot)

				If (dst\Friction > 0)

					fr# = 1-(dst\Friction)
					
					dst\VelX = dst\VelX * fr#
					dst\VelY = dst\VelY * fr#
					dst\VelZ = dst\VelZ * fr#

				EndIf

			EndIf

		Next

Return


.Animate_Camera

		; My chase cam

			te = car\player_pivot
			
			cyaw# = VectorYaw(EntityX(te,1) - EntityX(cam2), CamHeight + EntityY(te,1) - EntityY(cam2), EntityZ(te,1) - EntityZ(cam2))
			cpit# = VectorPitch(EntityX(te,1) - EntityX(cam2), CamHeight + EntityY(te,1) - EntityY(cam2), EntityZ(te,1) - EntityZ(cam2))

			maxpitch# = 70

			If (cpit > maxpitch) cpit = maxpitch
			If (cpit < -maxpitch) cpit = -maxpitch

			PositionEntity camnul, EntityX(te,1), EntityY(te,1), EntityZ(te,1)
			RotateEntity camnul, cpit, cyaw, 0
			MoveEntity camnul, 0, 0, -CamDist
			
			camy# = EntityY(camnul,1) + CamHeight
			
			If (camy < 5) Then camy = 5

			PositionEntity CamTarget, EntityX(camnul,1), camy, EntityZ(camnul,1)
			TranslateEntity cam2, (EntityX(CamTarget)-EntityX(Cam2))*CamLag, (EntityY(CamTarget)-EntityY(Cam2))*CamLag, (EntityZ(CamTarget)-EntityZ(Cam2))*CamLag
			TurnEntity cam2, DeltaPitch(cam2, te) * 0.5, DeltaYaw(cam2, te) * 0.5, -EntityRoll(cam2) * 0.05

		; My interior cam

			thisCP# = CloseAngle(EntityPitch(car\player, 1), prevCP)
			thisCY# = CloseAngle(EntityYaw(car\player, 1), prevCY)
			thisCR# = CloseAngle(EntityRoll(car\player, 1), prevCR)

			fr# = 0.3 ; camera friction
			afr# = 1.0 - fr
		
			RotateEntity (cam1, (thisCP*fr)+(prevCP*afr), (thisCY*fr)+(prevCY*afr), (thisCR*fr)+(prevCR*afr), 1)

			prevCP# = EntityPitch(cam1, 1)
			prevCY# = EntityYaw(cam1, 1)
			prevCR# = EntityRoll(cam1, 1)

Return